@@ -2,6 +2,7 @@
 """Tests the xonsh lexer."""
 from __future__ import unicode_literals, print_function
 import sys
+import ast
 import builtins
 import platform
 import subprocess
@@ -85,3 +86,23 @@ def check_eval(input):
 def check_parse(input):
     tree = builtins.__xonsh_execer__.parse(input, ctx=None)
     return tree
+
+#
+# Parser tools
+#
+
+
+def nodes_equal(x, y):
+    __tracebackhide__ = True
+    assert type(x) == type(y) , "Ast nodes do not have the same type: '%s' != '%s' " % (type(x), type(y))
+    if isinstance(x, (ast.Expr, ast.FunctionDef, ast.ClassDef)):
+        assert x.lineno == y.lineno, "Ast nodes do not have the same line number : %s != %s" % (x.lineno, y.lineno)
+        assert x.col_offset == y.col_offset, "Ast nodes do not have the same column offset number : %s != %s" % (x.col_offset, y.col_offset)
+    for (xname, xval), (yname, yval) in zip(ast.iter_fields(x),
+                                            ast.iter_fields(y)):
+        assert xname == yname, "Ast nodes fields differ : %s (of type %s) != %s (of type %s)" % (xname, type(xval), yname, type(yval))
+        assert type(xval) == type(yval), "Ast nodes fields differ : %s (of type %s) != %s (of type %s)" % (xname, type(xval), yname, type(yval))
+    for xchild, ychild in zip(ast.iter_child_nodes(x),
+                              ast.iter_child_nodes(y)):
+        assert nodes_equal(xchild, ychild) , "Ast node children differs"
+    return True
