@@ -1,24 +1,22 @@
 # -*- coding: utf-8 -*-
 """Tools to help interface with foreign shells, such as Bash."""
 import os
-import sys
 import re
 import json
 import shlex
+import sys
 import tempfile
 import builtins
 import subprocess
-from warnings import warn
-from functools import lru_cache
-from collections import MutableMapping, Mapping, Sequence
+import warnings
+import functools
+import collections.abc as cabc
 
-from xonsh.lazyasd import LazyObject
+from xonsh.lazyasd import lazyobject
 from xonsh.tools import to_bool, ensure_string
 from xonsh.platform import ON_WINDOWS, ON_CYGWIN
 
-
-COMMAND = """
-{seterrprevcmd}
+COMMAND = """{seterrprevcmd}
 {prevcmd}
 echo __XONSH_ENV_BEG__
 {envcmd}
@@ -30,11 +28,9 @@ echo __XONSH_FUNCS_BEG__
 {funcscmd}
 echo __XONSH_FUNCS_END__
 {postcmd}
-{seterrpostcmd}
-""".strip()
+{seterrpostcmd}"""
 
-DEFAULT_BASH_FUNCSCMD = r"""
-# get function names from declare
+DEFAULT_BASH_FUNCSCMD = r"""# get function names from declare
 declstr=$(declare -F)
 read -r -a decls <<< $declstr
 funcnames=""
@@ -66,10 +62,9 @@ if [[ "{" == "${namefile}" ]]; then
 else
   namefile="${namefile%?}}"
 fi
-echo $namefile
-""".strip()
+echo $namefile"""
 
-DEFAULT_ZSH_FUNCSCMD = """
+DEFAULT_ZSH_FUNCSCMD = """# get function names
 namefile="{"
 for name in ${(ok)functions}; do
   loc=$(whence -v $name)
@@ -82,11 +77,13 @@ if [[ "{" == "${namefile}" ]]; then
 else
   namefile="${namefile%?}}"
 fi
-echo ${namefile}
-""".strip()
+echo ${namefile}"""
+
 
 # mapping of shell name alises to keys in other lookup dictionaries.
-CANON_SHELL_NAMES = {
+@lazyobject
+def CANON_SHELL_NAMES():
+    return {
     'bash': 'bash',
     '/bin/bash': 'bash',
     'zsh': 'zsh',
@@ -94,51 +91,82 @@ CANON_SHELL_NAMES = {
     '/usr/bin/zsh': 'zsh',
     'cmd': 'cmd',
     'cmd.exe': 'cmd',
-}
+    }
 
-DEFAULT_ENVCMDS = {
+
+@lazyobject
+def DEFAULT_ENVCMDS():
+    return {
     'bash': 'env',
     'zsh': 'env',
     'cmd': 'set',
-}
-DEFAULT_ALIASCMDS = {
+    }
+
+
+@lazyobject
+def DEFAULT_ALIASCMDS():
+    return {
     'bash': 'alias',
     'zsh': 'alias -L',
     'cmd': '',
-}
-DEFAULT_FUNCSCMDS = {
+    }
+
+
+@lazyobject
+def DEFAULT_FUNCSCMDS():
+    return {
     'bash': DEFAULT_BASH_FUNCSCMD,
     'zsh': DEFAULT_ZSH_FUNCSCMD,
     'cmd': '',
-}
-DEFAULT_SOURCERS = {
+    }
+
+
+@lazyobject
+def DEFAULT_SOURCERS():
+    return {
     'bash': 'source',
     'zsh': 'source',
     'cmd': 'call',
-}
-DEFAULT_TMPFILE_EXT = {
+    }
+
+
+@lazyobject
+def DEFAULT_TMPFILE_EXT():
+    return {
     'bash': '.sh',
     'zsh': '.zsh',
     'cmd': '.bat',
-}
-DEFAULT_RUNCMD = {
+    }
+
+
+@lazyobject
+def DEFAULT_RUNCMD():
+    return {
     'bash': '-c',
     'zsh': '-c',
     'cmd': '/C',
-}
-DEFAULT_SETERRPREVCMD = {
+    }
+
+
+@lazyobject
+def DEFAULT_SETERRPREVCMD():
+    return {
     'bash': 'set -e',
     'zsh': 'set -e',
     'cmd': '@echo off',
-}
-DEFAULT_SETERRPOSTCMD = {
+    }
+
+
+@lazyobject
+def DEFAULT_SETERRPOSTCMD():
+    return {
     'bash': '',
     'zsh': '',
     'cmd': 'if errorlevel 1 exit 1',
-}
+    }
 
 
-@lru_cache()
+@functools.lru_cache()
 def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
                        aliascmd=None, extra_args=(), currenv=None,
                        safe=True, prevcmd='', postcmd='', funcscmd=None,
@@ -204,10 +232,10 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
     Returns
     -------
     env : dict
-        Dictionary of shell's environment
+        Dictionary of shell's environment. (None if the subproc command fails)
     aliases : dict
         Dictionary of shell's alaiases, this includes foreign function
-        wrappers.
+        wrappers.(None if the subproc command fails)
     """
     cmd = [shell]
     cmd.extend(extra_args)  # needs to come here for GNU long options
@@ -222,16 +250,14 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
     tmpfile_ext = DEFAULT_TMPFILE_EXT.get(shkey, 'sh') if tmpfile_ext is None else tmpfile_ext
     runcmd = DEFAULT_RUNCMD.get(shkey, '-c') if runcmd is None else runcmd
     seterrprevcmd = DEFAULT_SETERRPREVCMD.get(shkey, '') \
-                        if seterrprevcmd is None else seterrprevcmd
+        if seterrprevcmd is None else seterrprevcmd
     seterrpostcmd = DEFAULT_SETERRPOSTCMD.get(shkey, '') \
-                        if seterrpostcmd is None else seterrpostcmd
+        if seterrpostcmd is None else seterrpostcmd
     command = COMMAND.format(envcmd=envcmd, aliascmd=aliascmd, prevcmd=prevcmd,
                              postcmd=postcmd, funcscmd=funcscmd,
                              seterrprevcmd=seterrprevcmd,
                              seterrpostcmd=seterrpostcmd).strip()
-
     cmd.append(runcmd)
-
     if not use_tmpfile:
         cmd.append(command)
     else:
@@ -239,14 +265,14 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
         tmpfile.write(command.encode('utf8'))
         tmpfile.close()
         cmd.append(tmpfile.name)
-
     if currenv is None and hasattr(builtins, '__xonsh_env__'):
         currenv = builtins.__xonsh_env__.detype()
     elif currenv is not None:
         currenv = dict(currenv)
     try:
         s = subprocess.check_output(cmd, stderr=subprocess.PIPE, env=currenv,
-                                    # start new session to avoid hangs (doesn't work on Cygwin though)
+                                    # start new session to avoid hangs
+                                    # (doesn't work on Cygwin though)
                                     start_new_session=(not ON_CYGWIN),
                                     universal_newlines=True)
     except (subprocess.CalledProcessError, FileNotFoundError):
@@ -263,12 +289,16 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
     return env, aliases
 
 
-ENV_RE = LazyObject(lambda: re.compile('__XONSH_ENV_BEG__\n(.*)'
-                                       '__XONSH_ENV_END__', flags=re.DOTALL),
-                    globals(), 'ENV_RE')
-ENV_SPLIT_RE = LazyObject(lambda: re.compile('^([^=]+)=([^=]*|[^\n]*)$',
-                                             flags=re.DOTALL|re.MULTILINE),
-                          globals(), 'ENV_SPLIT_RE')
+@lazyobject
+def ENV_RE():
+    return re.compile('__XONSH_ENV_BEG__\n(.*)'
+                      '__XONSH_ENV_END__', flags=re.DOTALL)
+
+
+@lazyobject
+def ENV_SPLIT_RE():
+    return re.compile('^([^=]+)=([^=]*|[^\n]*)$',
+                      flags=re.DOTALL | re.MULTILINE)
 
 
 def parse_env(s):
@@ -282,7 +312,10 @@ def parse_env(s):
     return env
 
 
-ALIAS_RE = re.compile('__XONSH_ALIAS_BEG__\n(.*)__XONSH_ALIAS_END__',
+@lazyobject
+def ALIAS_RE():
+    return re.compile('__XONSH_ALIAS_BEG__\n(.*)'
+                      '__XONSH_ALIAS_END__',
                       flags=re.DOTALL)
 
 
@@ -305,14 +338,17 @@ def parse_aliases(s):
                 value = value[1:-1]
             value = shlex.split(value)
         except ValueError as exc:
-            warn('could not parse alias "{0}": {1!r}'.format(key, exc),
-                 RuntimeWarning)
+            warnings.warn('could not parse alias "{0}": {1!r}'.format(key, exc),
+                          RuntimeWarning)
             continue
         aliases[key] = value
     return aliases
 
 
-FUNCS_RE = re.compile('__XONSH_FUNCS_BEG__\n(.+)\n__XONSH_FUNCS_END__',
+@lazyobject
+def FUNCS_RE():
+    return re.compile('__XONSH_FUNCS_BEG__\n(.+)\n'
+                      '__XONSH_FUNCS_END__',
                       flags=re.DOTALL)
 
 
@@ -335,14 +371,14 @@ def parse_funcs(s, shell, sourcer=None):
                'Note: you may be seeing this error if you use zsh with '
                'prezto. Prezto overwrites GNU coreutils functions (like echo) '
                'with its own zsh functions. Please try disabling prezto.')
-        warn(msg.format(exc, shell, s, g1), RuntimeWarning)
+        warnings.warn(msg.format(exc, shell, s, g1), RuntimeWarning)
         return {}
     sourcer = DEFAULT_SOURCERS.get(shell, 'source') if sourcer is None \
-                                                    else sourcer
+        else sourcer
     funcs = {}
     for funcname, filename in namefiles.items():
-        if funcname.startswith('_'):
-            continue  # skip private functions
+        if funcname.startswith('_') or not filename:
+            continue  # skip private functions and invalid files
         if not os.path.isabs(filename):
             filename = os.path.abspath(filename)
         wrapper = ForeignShellFunctionAlias(name=funcname, shell=shell,
@@ -373,7 +409,7 @@ class ForeignShellFunctionAlias(object):
             Command to source foreing files with.
         """
         sourcer = DEFAULT_SOURCERS.get(shell, 'source') if sourcer is None \
-                                                        else sourcer
+            else sourcer
         self.name = name
         self.shell = shell
         self.filename = filename
@@ -381,7 +417,7 @@ class ForeignShellFunctionAlias(object):
 
     def __eq__(self, other):
         if not hasattr(other, 'name') or not hasattr(other, 'shell') or \
-           not hasattr(other, 'filename') or not hasattr(other, 'sourcer'):
+                not hasattr(other, 'filename') or not hasattr(other, 'sourcer'):
             return NotImplemented
         return (self.name == other.name) and (self.shell == other.shell) and \
                (self.filename == other.filename) and (self.sourcer == other.sourcer)
@@ -412,13 +448,18 @@ class ForeignShellFunctionAlias(object):
         return args, True
 
 
-VALID_SHELL_PARAMS = frozenset(['shell', 'interactive', 'login', 'envcmd',
-                                'aliascmd', 'extra_args', 'currenv', 'safe',
-                                'prevcmd', 'postcmd', 'funcscmd', 'sourcer'])
+@lazyobject
+def VALID_SHELL_PARAMS():
+    return frozenset([
+    'shell', 'interactive', 'login', 'envcmd',
+    'aliascmd', 'extra_args', 'currenv', 'safe',
+    'prevcmd', 'postcmd', 'funcscmd', 'sourcer',
+    ])
+
 
 def ensure_shell(shell):
     """Ensures that a mapping follows the shell specification."""
-    if not isinstance(shell, MutableMapping):
+    if not isinstance(shell, cabc.MutableMapping):
         shell = dict(shell)
     shell_keys = set(shell.keys())
     if not (shell_keys <= VALID_SHELL_PARAMS):
@@ -431,17 +472,17 @@ def ensure_shell(shell):
         shell['login'] = to_bool(shell['login'])
     if 'envcmd' in shell_keys:
         shell['envcmd'] = None if shell['envcmd'] is None \
-                               else ensure_string(shell['envcmd'])
+            else ensure_string(shell['envcmd'])
     if 'aliascmd' in shell_keys:
         shell['aliascmd'] = None if shell['aliascmd'] is None \
-                                 else ensure_string(shell['aliascmd'])
+            else ensure_string(shell['aliascmd'])
     if 'extra_args' in shell_keys and not isinstance(shell['extra_args'], tuple):
         shell['extra_args'] = tuple(map(ensure_string, shell['extra_args']))
     if 'currenv' in shell_keys and not isinstance(shell['currenv'], tuple):
         ce = shell['currenv']
-        if isinstance(ce, Mapping):
+        if isinstance(ce, cabc.Mapping):
             ce = tuple([(ensure_string(k), v) for k, v in ce.items()])
-        elif isinstance(ce, Sequence):
+        elif isinstance(ce, cabc.Sequence):
             ce = tuple([(ensure_string(k), v) for k, v in ce])
         else:
             raise RuntimeError('unrecognized type for currenv')
@@ -454,16 +495,16 @@ def ensure_shell(shell):
         shell['postcmd'] = ensure_string(shell['postcmd'])
     if 'funcscmd' in shell_keys:
         shell['funcscmd'] = None if shell['funcscmd'] is None \
-                                 else ensure_string(shell['funcscmd'])
+            else ensure_string(shell['funcscmd'])
     if 'sourcer' in shell_keys:
         shell['sourcer'] = None if shell['sourcer'] is None \
-                                 else ensure_string(shell['sourcer'])
+            else ensure_string(shell['sourcer'])
     if 'seterrprevcmd' in shell_keys:
         shell['seterrprevcmd'] = None if shell['seterrprevcmd'] is None \
-                                 else ensure_string(shell['seterrprevcmd'])
+            else ensure_string(shell['seterrprevcmd'])
     if 'seterrpostcmd' in shell_keys:
         shell['seterrpostcmd'] = None if shell['seterrpostcmd'] is None \
-                                 else ensure_string(shell['seterrpostcmd'])
+            else ensure_string(shell['seterrpostcmd'])
     return shell
 
 
@@ -535,9 +576,18 @@ def load_foreign_aliases(shells=None, config=None, issue_warning=True):
     """
     shells = _get_shells(shells=shells, config=config, issue_warning=issue_warning)
     aliases = {}
+    xonsh_aliases = builtins.aliases
     for shell in shells:
         shell = ensure_shell(shell)
         _, shaliases = foreign_shell_data(**shell)
-        if shaliases:
-            aliases.update(shaliases)
+        if not builtins.__xonsh_env__.get('FOREIGN_ALIASES_OVERRIDE'):
+            shaliases = {} if shaliases is None else shaliases
+            for alias in set(shaliases) & set(xonsh_aliases):
+                del shaliases[alias]
+                if builtins.__xonsh_env__.get('XONSH_DEBUG'):
+                    print('aliases: ignoring alias {!r} of shell {!r} '
+                          'which tries to override xonsh alias.'
+                          ''.format(alias, shell['shell']),
+                          file=sys.stderr)
+        aliases.update(shaliases)
     return aliases
