@@ -4,6 +4,7 @@ import os
 import re
 import string
 import builtins
+import functools
 from warnings import warn
 from collections import ChainMap
 from collections.abc import MutableMapping
@@ -11,14 +12,11 @@ from collections.abc import MutableMapping
 # must come before pygments imports
 from xonsh.lazyasd import load_module_in_background
 
-load_module_in_background('pkg_resources', debug='XONSH_DEBUG',
-                          replacements={'pygments.plugin': 'pkg_resources'})
-
-from pygments.lexer import inherit, bygroups, using, this
-from pygments.lexers.shell import BashLexer
+from pygments.lexer import inherit, bygroups, include
 from pygments.lexers.agile import PythonLexer
 from pygments.token import (Keyword, Name, Comment, String, Error, Number,
-                            Operator, Generic, Whitespace, Token)
+                            Operator, Generic, Whitespace, Token, Punctuation,
+                            Text)
 from pygments.style import Style
 from pygments.styles import get_style_by_name
 import pygments.util
@@ -29,28 +27,40 @@ from xonsh.tools import (ON_WINDOWS, intensify_colors_for_cmd_exe,
 from xonsh.tokenize import SearchPath
 
 
-class XonshSubprocLexer(BashLexer):
-    """Lexer for xonsh subproc mode."""
-    name = 'Xonsh subprocess lexer'
-    tokens = {'root': [(SearchPath, String.Backtick), inherit, ]}
+load_module_in_background('pkg_resources', debug='XONSH_DEBUG',
+                          replacements={'pygments.plugin': 'pkg_resources'})
 
 
-ROOT_TOKENS = [(r'\?', Keyword),
-               (r'\$\w+', Name.Variable),
-               (r'\$\{', Keyword, ('pymode', )),
-               (r'[\!\$]\(', Keyword, ('subproc', )),
-               (r'[\!\$]\[', Keyword, ('subproc', )),
-               (r'@\$\(', Keyword, ('subproc', )),
-               (r'@\(', Keyword, ('pymode', )),
-               inherit, ]
+def _command_is_valid(cmd):
+    cmd_abspath = os.path.abspath(os.path.expanduser(cmd))
+    return cmd in builtins.__xonsh_commands_cache__ or \
+        (os.path.isfile(cmd_abspath) and os.access(cmd_abspath, os.X_OK))
 
-PYMODE_TOKENS = [(r'(.+)(\))', bygroups(using(this), Keyword), '#pop'),
-                 (r'(.+)(\})', bygroups(using(this), Keyword), '#pop'), ]
 
-SUBPROC_TOKENS = [
-    (r'(.+)(\))', bygroups(using(XonshSubprocLexer), Keyword), '#pop'),
-    (r'(.+)(\])', bygroups(using(XonshSubprocLexer), Keyword), '#pop'),
-]
+def _command_is_autocd(cmd):
+    if not builtins.__xonsh_env__.get('AUTO_CD'):
+        return False
+    cmd_abspath = os.path.abspath(os.path.expanduser(cmd))
+    return os.path.isdir(cmd_abspath)
+
+
+def subproc_cmd_callback(_, match):
+    """Yield Builtin token if match contains valid command,
+    otherwise fallback to fallback lexer.
+    """
+    cmd = match.group()
+    yield match.start(), Name.Builtin if _command_is_valid(cmd) else Error, cmd
+
+
+def subproc_arg_callback(_, match):
+    """Check if match contains valid path"""
+    text = match.group()
+    yield (match.start(),
+           Name.Constant if os.path.exists(os.path.expanduser(text)) else Text,
+           text)
+
+
+COMMAND_TOKEN_RE = r'[^=\s\[\]{}()$"\'`<&|;!]+(?=\s|$|\)|\]|\}|!)'
 
 
 class XonshLexer(PythonLexer):
@@ -60,14 +70,105 @@ class XonshLexer(PythonLexer):
     aliases = ['xonsh', 'xsh']
     filenames = ['*.xsh', '*xonshrc']
 
+    def __init__(self, *args, **kwargs):
+        _ = builtins.__xonsh_commands_cache__.all_commands
+        super().__init__(*args, **kwargs)
+
     tokens = {
-        'root': list(ROOT_TOKENS),
-        'pymode': PYMODE_TOKENS,
-        'subproc': SUBPROC_TOKENS,
+        'mode_switch_brackets': [
+            (r'(\$)(\{)', bygroups(Keyword, Punctuation), 'py_curly_bracket'),
+            (r'(@)(\()', bygroups(Keyword, Punctuation), 'py_bracket'),
+            (r'([\!\$])(\()', bygroups(Keyword, Punctuation),
+             ('subproc_bracket', 'subproc_start')),
+            (r'(@\$)(\()', bygroups(Keyword, Punctuation),
+             ('subproc_bracket', 'subproc_start')),
+            (r'([\!\$])(\[)', bygroups(Keyword, Punctuation),
+             ('subproc_square_bracket', 'subproc_start')),
+            (r'(g?)(`)', bygroups(String.Affix, String.Backtick),
+             'backtick_re'),
+        ],
+        'subproc_bracket': [
+            (r'\)', Punctuation, '#pop'),
+            include('subproc'),
+        ],
+        'subproc_square_bracket': [
+            (r'\]', Punctuation, '#pop'),
+            include('subproc'),
+        ],
+        'py_bracket': [
+            (r'\)', Punctuation, '#pop'),
+            include('root'),
+        ],
+        'py_curly_bracket': [
+            (r'\}', Punctuation, '#pop'),
+            include('root'),
+        ],
+        'backtick_re': [
+            (r'[\.\^\$\*\+\?\[\]\|]', String.Regex),
+            (r'({[0-9]+}|{[0-9]+,[0-9]+})\??', String.Regex),
+            (r'\\([0-9]+|[AbBdDsSwWZabfnrtuUvx\\])', String.Escape),
+            (r'`', String.Backtick, '#pop'),
+            (r'[^`\.\^\$\*\+\?\[\]\|]+', String.Backtick),
+        ],
+        'root': [
+            (r'\?', Keyword),
+            (r'(?<=\w)!', Keyword),
+            (r'\$\w+', Name.Variable),
+            (r'\(', Punctuation, 'py_bracket'),
+            (r'\{', Punctuation, 'py_curly_bracket'),
+            include('mode_switch_brackets'),
+            inherit,
+        ],
+        'subproc_start': [
+            (r'\s+', Whitespace),
+            (COMMAND_TOKEN_RE, subproc_cmd_callback, '#pop'),
+            (r'', Whitespace, '#pop'),
+        ],
+        'subproc': [
+            include('mode_switch_brackets'),
+            (r'&&|\|\|', Operator, 'subproc_start'),
+            (r'"(\\\\|\\[0-7]+|\\.|[^"\\])*"', String.Double),
+            (r"'(\\\\|\\[0-7]+|\\.|[^'\\])*'", String.Single),
+            (r'(?<=\w|\s)!', Keyword, 'subproc_macro'),
+            (r'^!', Keyword, 'subproc_macro'),
+            (r';', Punctuation, 'subproc_start'),
+            (r'&|=', Punctuation),
+            (r'\|', Punctuation, 'subproc_start'),
+            (r'\s+', Text),
+            (r'[^=\s\[\]{}()$"\'`<&|;]+', subproc_arg_callback),
+            (r'<', Text),
+            (r'\$\w+', Name.Variable),
+        ],
+        'subproc_macro': [
+            (r'(\s*)([^\n]+)', bygroups(Whitespace, String)),
+            (r'', Whitespace, '#pop'),
+        ],
     }
 
+    def get_tokens_unprocessed(self, text):
+        """Check first command, then call super.get_tokens_unprocessed
+        with root or subproc state"""
+        start = 0
+        state = ('root', )
+        m = re.match(r'(\s*)({})'.format(COMMAND_TOKEN_RE), text)
+        if m is not None:
+            yield m.start(1), Whitespace, m.group(1)
+            cmd = m.group(2)
+            cmd_is_valid = _command_is_valid(cmd)
+            cmd_is_autocd = _command_is_autocd(cmd)
 
-class XonshConsoleLexer(PythonLexer):
+            if cmd_is_valid or cmd_is_autocd:
+                yield (m.start(2),
+                       Name.Builtin if cmd_is_valid else Name.Constant,
+                       cmd)
+                start = m.end(2)
+                state = ('subproc', )
+
+        for i, t, v in super().get_tokens_unprocessed(text[start:], state):
+            yield i + start, t, v
+
+
+class XonshConsoleLexer(XonshLexer):
     """Xonsh console lexer for pygments."""
 
     name = 'Xonsh console lexer'
@@ -78,17 +179,10 @@ class XonshConsoleLexer(PythonLexer):
         'root': [(r'^(>>>|\.\.\.) ', Generic.Prompt),
                  (r'\n(>>>|\.\.\.)', Generic.Prompt),
                  (r'\n(?![>.][>.][>.] )([^\n]*)', Generic.Output),
-                 (r'\n(?![>.][>.][>.] )(.*?)$', Generic.Output)] + ROOT_TOKENS,
-        'pymode': PYMODE_TOKENS,
-        'subproc': SUBPROC_TOKENS,
+                 (r'\n(?![>.][>.][>.] )(.*?)$', Generic.Output),
+                 inherit]
     }
 
-# XonshLexer & XonshSubprocLexer have to reference each other
-XonshSubprocLexer.tokens['root'] = [
-    (r'(\$\{)(.*)(\})', bygroups(Keyword, using(XonshLexer), Keyword)),
-    (r'(@\()(.+)(\))', bygroups(Keyword, using(XonshLexer), Keyword)),
-] + XonshSubprocLexer.tokens['root']
-
 
 #
 # Colors and Styles
